package it.mokabyte.pattern.client;

import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import java.rmi.RemoteException;

import it.mokabyte.pattern.ejb.session.stateful.FacadeHome;
import it.mokabyte.pattern.ejb.session.stateful.Facade;

import it.mokabyte.pattern.valueobject.ProductOM;

import it.mokabyte.pattern.util.JndiNames;

/**
 * <p>Title:AccountTestClient </p>
 * <p>Description: simple client that <b>use</b> Facade pattern </p>
 * Test will be OK if the first two products can be bought but the third one not !</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S. - Dozio L.
 * @version 1.0
 */
public class FacadeTestClient {

    /** main */
    public static void main(String[] args) {
        System.out.println("starting FacadeTestClient ...");
        if(args.length == 1) {
          new FacadeTestClient().test(args[0]);
        }
        else {
          System.out.println("usage : FacadeTestClient <USER_ID>");
        }
    }

    /** Test the Entity EJB Account */
    public void test(String userid) {
      try {
          Context ctx = null;
          ctx =  new InitialContext();
          System.out.println("Lookup for ["+JndiNames.FACADE+"]...");
          //look up jndi name
          Object obj = ctx.lookup(JndiNames.FACADE);
          System.out.println("Lookup done : obj class["+obj.getClass().getName()+"] ...");
          System.out.println("Lookup done : obj ["+obj+"] ...");
          //cast to Home interface
          FacadeHome facadeHome = (FacadeHome)PortableRemoteObject.narrow(obj, FacadeHome.class);
          System.out.println("Narrow done ...facadeHome["+facadeHome+"]...");
          // start the test
          // find the account
          Facade facade = facadeHome.create(userid);

          // read data
          double balance = facade.getBalance();
          System.out.println("User account balance : " + balance);

          facade.addItem(new ProductOM("Product1",100));
          System.out.println("Acquisto 1 possibile...");
          facade.addItem(new ProductOM("Product2",500));
          System.out.println("Acquisto 2 possibile...");
          facade.addItem(new ProductOM("Product2",500));
          System.out.println("Acquisto 3 possibile...");

        } catch(ClassCastException cce) {
            System.out.println(" # ClassCastExceptionException : " + cce.getMessage());
            cce.printStackTrace();
        } catch(NamingException ne) {
            System.out.println(" # NamingException : " + ne.getMessage());
            ne.printStackTrace();
        } catch(CreateException ce) {
            System.out.println(" # CreateException : " + ce.getMessage());
            ce.printStackTrace();
        } catch(RemoteException re) {
            System.out.println(" # RemoteException : " + re.getMessage());
            re.printStackTrace();
        }
        catch(Exception e) {
            System.out.println(" # Exception  : " + e.getMessage());
            e.printStackTrace();
        }
    }
}