package it.mokabyte.pattern.client;

//import from j2sde distribution
import java.io.IOException;

//import from previous mokabyte
import it.mokabyte.pattern.ejb.entity.AccountHome;
import it.mokabyte.pattern.ejb.session.stateful.ShopperHome;
import it.mokabyte.pattern.ejb.session.stateful.Shopper;
import it.mokabyte.pattern.ejb.session.stateful.FacadeHome;
import it.mokabyte.pattern.util.JndiNames;
import it.mokabyte.pattern.locator.MyServiceLocator;
import it.mokabyte.pattern.exception.ServiceLocationException;

//import freom j2ee distribution
import javax.jms.TopicConnectionFactory;
import javax.ejb.EJBObject;

//import from junit
import junit.framework.TestCase;
import junit.framework.TestSuite;
//import junit.swingui.TestRunner;
import junit.framework.Test;
import junit.textui.TestRunner;

//import util
import it.mokabyte.pattern.util.J2EEPatternTestResult;

/**
 * <p>Title:JUServiceLocatorTestClient </p>
 * <p>Description: simple client that <b>use</b> service locator pattern</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author :Dozio L.
 * @version 1.0
 */
public class JUServiceLocatorTestClient extends TestCase{


    public JUServiceLocatorTestClient(String testName){
      super(testName);
      System.out.println("JUServiceLocatorTestClient");
    }

    /**
     * Method to test looking up on SunRi application server
     */
    public void testLookingUpFromSunRi() {
      // 1. Locating Account EJB from SUN RI using type checked API of ServiceLocator
      try {
        System.out.println("testLookingUpFromSunRi ...");
        AccountHome home = (AccountHome)MyServiceLocator.getInstance().getHome(MyServiceLocator.MOKA_ACCOUNT_SERVICE);
        System.out.println("home class["+home.getClass().getName()+"] ...");
        // System.out.println("home ["+home+"] ...");
        this.assertNotNull(home);
      } catch(ClassCastException cce) {
        this.fail("ClassCastException: " + cce.getMessage());

      } catch(ServiceLocationException sle) {
        this.fail(" ServiceLocationException: " + sle.getMessage());
      }
      catch(Exception e) {
        this.fail(" Generic Exception: " + e.getMessage());
      }
      }
    /**
     * Method to test looking up on JBoss application server
     */
    public void testLookingUpFromJBoss() {
      try {
       System.out.println("testLookingUpFromJBoss ...");
       Object o = MyServiceLocator.getInstance().getHome(MyServiceLocator.MOKA_SHOPPER_SERVICE);
       System.out.println("oooooooooooooooooooooooooo class["+o.getClass().getName()+"] ...");
        ShopperHome home = (ShopperHome)o;
        System.out.println("home class["+home.getClass().getName()+"] ...");
        this.assertNotNull(home);

      } catch(ClassCastException cce) {
        this.fail("ClassCastException: " + cce.getMessage());

      } catch(ServiceLocationException sle) {
        this.fail(" ServiceLocationException: " + sle.getMessage());
      }
      catch(Exception e) {
        this.fail(" Generic Exception: " + e.getMessage());
      }
      }

     /**
     * Method to test looking up on JBoss application server with uncheked API
     */
    public void testLookingUpFromJBossWithUncheckedApi() {
    try {
          System.out.println("testLookingUpFromJBossWithUncheckedApi ...");
          FacadeHome home = (FacadeHome)MyServiceLocator.getInstance().getHome(JndiNames.FACADE,FacadeHome.class);
          System.out.println("home class["+home.getClass().getName()+"] ...");
          // System.out.println("home ["+home+"] ...");
           this.assertNotNull(home);

      } catch(ClassCastException cce) {
        this.fail("ClassCastException: " + cce.getMessage());

      } catch(ServiceLocationException sle) {
        this.fail(" ServiceLocationException: " + sle.getMessage());
      }
      catch(Exception e) {
        this.fail(" Generic Exception: " + e.getMessage());
      }
      }


    /**
     * Method to test JMS Topic looking up on JBoss application server
     */
    public void testLookingUpFromJBossJmsTopic() {
      // 4. Locating JMS Topic from JBoss using type UNchecked API of ServiceLocator
      try {
           System.out.println("testLookingUpFromJBossJmsTopic ...");
          TopicConnectionFactory home = (TopicConnectionFactory)MyServiceLocator.getInstance().getTopicHome(JndiNames.TOPIC_CONNECTION_FACTORY);
          System.out.println("home class["+home.getClass().getName()+"] ...");
          this.assertNotNull("TopicConnectionFactory class is null!",home);
      } catch(ClassCastException cce) {
        this.fail("ClassCastException: " + cce.getMessage());
      } catch(ServiceLocationException sle) {
        this.fail(" ServiceLocationException: " + sle.getMessage());
      }
      catch(Exception e) {
        this.fail(" Generic Exception: " + e.getMessage());
      }
      }
    /**
     * suite di test with all tets for ServiceLocator
     * @return suite test
     */

     public static Test suite(){
      TestSuite suite = new TestSuite("ServiceLocatorTest");
      suite.addTest(new JUServiceLocatorTestClient("testLookingUpFromSunRi"));
      suite.addTest(new JUServiceLocatorTestClient("testLookingUpFromJBoss"));
      suite.addTest(new JUServiceLocatorTestClient("testLookingUpFromJBossWithUncheckedApi"));
      suite.addTest(new JUServiceLocatorTestClient("testLookingUpFromJBossJmsTopic"));
      return suite;

     }


      /** main */
      public static void main (String[] args) {
		if(args.length!=0) {
			if (args[0].equalsIgnoreCase("swing")) {
                                System.out.println("Runnign SWING Graphical User Interface ...");
                                junit.swingui.TestRunner.run (J2EEPatternTestResult.class);
                        }
			if (args[0].equalsIgnoreCase("xml")){
				J2EEPatternTestResult j2eeTr = new J2EEPatternTestResult(suite());
				suite().run(j2eeTr);
				System.out.println(j2eeTr);
				try{
					java.io.FileOutputStream fos = new java.io.FileOutputStream("./ServiceLocatorTestResult.xml");
                                        fos.write(j2eeTr.toString().getBytes());
					fos.flush();
				}
				catch (Exception e){
					e.printStackTrace(System.err);
				}
			}
		}
		else {
                    System.out.println("Runnign TEXT User Interface ...");
                    junit.textui.TestRunner.run (suite());
                }
	}


}