package it.mokabyte.pattern.client;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import it.mokabyte.pattern.ejb.session.stateful.FacadeHome;
import it.mokabyte.pattern.ejb.session.stateful.Facade;
import it.mokabyte.pattern.util.JndiNames;
import it.mokabyte.pattern.valueobject.ProductOM;
import it.mokabyte.pattern.exception.BusinessException;

/**
 * <p>Title:BusinessDelegateTestClient </p>
 * <p>Description: simple client that <b>NOT</b> use Business Delegate pattern </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S. - Dozio L.
 * @version 1.0
 */
public class NoBusinessDelegateTestClient {

    /** main */
    public static void main(String[] args) {
        System.out.println("starting NoBusinessDelegateTestClient ...");
        if(args.length == 1) {
          new NoBusinessDelegateTestClient().test(args[0]);
        }
        else {
          System.out.println("usage : NoBusinessDelegateTestClient <USER_ID>");
        }
    }

    /**
     * <li> use MyBusinessDelegate to use the "hidden" Facade
     * <li> invoke business methods
     * @param userid
     */
    public void test(String userid) {
      try {
        Context ctx = new InitialContext();
        Object result = ctx.lookup(JndiNames.FACADE);
        FacadeHome home = (FacadeHome)PortableRemoteObject.narrow(result, FacadeHome.class);

        System.out.println("NoBusinessDelegateTestClient : home class["+home.getClass().getName()+"] ...");
        System.out.println("NoBusinessDelegateTestClient : home ["+home+"] ...");

        Facade remote = home.create(userid);
        System.out.println("Amount : " + remote.getAmount());
      }
      catch(NamingException ne) {
            System.out.println(" # Exception  : " + ne.getMessage());
            ne.printStackTrace();
      }
      catch(RemoteException re) {
            System.out.println(" # Exception  : " + re.getMessage());
            re.printStackTrace();
      }
      catch(CreateException ce) {
            System.out.println(" # Exception  : " + ce.getMessage());
            ce.printStackTrace();
      }
      catch(BusinessException be) {
            System.out.println(" # Exception  : " + be.getMessage());
            be.printStackTrace();
      }

    }
}