package it.mokabyte.pattern.client;

import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import java.rmi.RemoteException;

import it.mokabyte.pattern.ejb.session.stateful.ShopperHome;
import it.mokabyte.pattern.ejb.session.stateful.Shopper;

import it.mokabyte.pattern.ejb.entity.Account;
import it.mokabyte.pattern.ejb.entity.AccountHome;
import it.mokabyte.pattern.pk.AccountPK;

import it.mokabyte.pattern.valueobject.AccountOM;
import it.mokabyte.pattern.valueobject.ProductOM;

import it.mokabyte.pattern.util.JndiNames;

/**
 * <p>Title:NoFacadeTestClient </p>
 * <p>Description: simple client that <b>NOT</b> use Facade pattern </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S. - Dozio L.
 * @version 1.0
 */
public class NoFacadeTestClient {

    /** main */
    public static void main(String[] args) {
        System.out.println("starting NoFacadeTestClient ...");
        if(args.length == 1) {
          new NoFacadeTestClient().test(args[0]);
        }
        else {
          System.out.println("usage : NoFacadeTestClient <USER_ID>");
        }
    }

    /** Test the Entity EJB Account */
    public void test(String userid) {
      try {
          Context ctx = null;
          ctx =  new InitialContext();
          System.out.println("Lookup for ["+JndiNames.ACCOUNT+"]...");
          //look up jndi name
          Object obj = ctx.lookup(JndiNames.ACCOUNT);
          System.out.println("Lookup done : obj class["+obj.getClass().getName()+"] ...");
          System.out.println("Lookup done : obj ["+obj+"] ...");
          //cast to Home interface
          AccountHome accountHome = (AccountHome)PortableRemoteObject.narrow(obj, AccountHome.class);
          System.out.println("Narrow done ...accountHome["+accountHome+"]...");
          // start the test
          // find the account
          Account account = accountHome.findByPrimaryKey(new AccountPK(userid));

          // read data
          double balance = account.getBalance();
          System.out.println("User account balance : " + balance);

          obj = ctx.lookup(JndiNames.SHOPPER);
          System.out.println("before narrowing of ["+obj.getClass().getName()+"]...");
          ShopperHome shopperHome  = (ShopperHome)javax.rmi.PortableRemoteObject.narrow(obj,ShopperHome.class);
          System.out.println("shopperHome ["+shopperHome.getClass().getName()+"]");

          Shopper shopper = shopperHome.create(userid);
          System.out.println("create Shopper done !");

          shopper.addItem(new ProductOM("Prodotto1",10));
          shopper.addItem(new ProductOM("Prodotto2",20));

          if(shopper.getAmount() > account.getBalance())
            System.out.println("Acquisto # NON # possibile !");
          else
            System.out.println("Acquisto possibile.");

        } catch(ClassCastException cce) {
            System.out.println(" # ClassCastExceptionException : " + cce.getMessage());
            cce.printStackTrace();
        } catch(NamingException ne) {
            System.out.println(" # NamingException : " + ne.getMessage());
            ne.printStackTrace();
        } catch(CreateException ce) {
            System.out.println(" # CreateException : " + ce.getMessage());
            ce.printStackTrace();
        } catch(FinderException fe) {
            System.out.println(" # FinderException : " + fe.getMessage());
            fe.printStackTrace();
        } catch(RemoteException re) {
            System.out.println(" # RemoteException : " + re.getMessage());
            re.printStackTrace();
        }
        catch(Exception e) {
            System.out.println(" # Exception  : " + e.getMessage());
            e.printStackTrace();
        }
    }
}