package it.mokabyte.pattern.dao;

import javax.ejb.*;
import java.util.*;
import javax.naming.* ;

import it.mokabyte.pattern.pk.AccountPK;
import it.mokabyte.pattern.exception.DAOException;
import it.mokabyte.pattern.valueobject.AccountOM;

/**
 * <p>Title: MyDao </p>
 * <p>Description: the dao interface</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S. - Dozio L.
 * @version 1.0
 */
public interface MyDao {

    /**
     * Insert an Account
     * @return the number of rows inserted
     * @param pk primary key for the account
     * @param name the account user name
     * @param address the account user address
     * @param balance the initial balance for this account
     * @exception DAOException a data access error occured
     */
    public int insertAccount(AccountPK pk, String name, String address, double balance) throws DAOException;

    /**
     * Delete an Account
     * @return the number of rows deleted
     * @param  the primary key
     * @exception DAOException a data access error occured
     */
    public int deleteAccount(AccountPK pk) throws DAOException;

    /**
     * Get an Account
     * @return the ObjectModel of the Account
     * @param  the primary key
     * @exception DAOException a data access error occured
     */
    public AccountOM readAccount(AccountPK pk) throws DAOException;

    /**
     * Update an account
     * @return the number of rows updated
     * @param key the primary key
     * @param name user name
     * @param address user address
     * @param balance user balance
     * @exception DAOException a data access error occured
     */
    public int updateAccount(AccountPK key, String name, String address, double balance) throws DAOException;

    /**
     * @return true if account exists
     * @param key the Primary Key for this account
     * @exception DAOException a data access error occured
     */
     public boolean accountExists(AccountPK key) throws DAOException;
}