package it.mokabyte.pattern.dao;

import java.sql.*;
import javax.ejb.*;
import java.util.*;
import javax.sql.* ;
import javax.naming.* ;

import it.mokabyte.pattern.util.SimpleTrace;

import it.mokabyte.pattern.exception.DAOException;

import it.mokabyte.pattern.dao.MyDaoJdbcImpl;
import it.mokabyte.pattern.dao.MyDaoXmlImpl;

/**
 * <p>Title: MyDaoFactory </p>
 * <p>Description: the dao factory class</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S. - Dozio L.
 * @version 1.0
 */
public class MyDaoFactory {

   /** Constants of DAO type */
   public static final int XML_TYPE = 1;
   public static final int JDBC_TYPE = 2;

   /**
    * <p> Instantiate the class specified by Deployment descriptor environment entry </p>
    * @return the Dao class
    * @throws DAOException
    */
    public static MyDao getDAO() throws DAOException {

        MyDao dao = null;
        try {
            InitialContext ic = new InitialContext();
            SimpleTrace.trace("MyDAOFactory.getDao : looking for ["+DaoConfig.DAO_CLASS_NAME+"]...");
            String className = (String) ic.lookup(DaoConfig.DAO_CLASS_NAME);
            SimpleTrace.trace("MyDAOFactory.getDao : className ["+className+"]...");
            dao = (MyDao) Class.forName(className).newInstance();
        } catch (ClassNotFoundException cnfe) {
            SimpleTrace.trace("MyDAOFactory.getDao : " + cnfe.getMessage());
            cnfe.printStackTrace();
            throw new DAOException(cnfe);
        } catch (ClassCastException cce) {
            SimpleTrace.trace("MyDAOFactory.getDao : " + cce.getMessage());
            cce.printStackTrace();
            throw new DAOException(cce);
       } catch (NamingException ne) {
            SimpleTrace.trace("MyDAOFactory.getDao : " + ne.getMessage());
            ne.printStackTrace();
            throw new DAOException("MyDAOFactory.getDAO: NamingException : " + ne.getMessage());
        } catch (Exception e) {
            SimpleTrace.trace("MyDAOFactory.getDao : " + e.getMessage());
            e.printStackTrace();
            throw new DAOException("MyDAOFactory.getDAO: Exception while getting DAO type :" + e.getMessage());
        }

        SimpleTrace.trace("MyDAOFactory.getDao : returning class ["+dao.getClass().getName()+"]...");
        return dao;
    }

   /**
    * <p> Instantiate the class specified by user </p>
    * @return the Dao class
    * @throws DAOException
    */
    public static MyDao getDAO(int DaoType) throws DAOException {

        MyDao dao = null;

        try {
            switch(DaoType) {
                case JDBC_TYPE :
                  dao = new MyDaoJdbcImpl();
                break;

                case XML_TYPE :
                  dao = new MyDaoXmlImpl();
                break;

                default :
                  throw new DAOException("MyDAOFactory.getDAO: ["+DaoType+"] is an UNKNOWN TYPE !");
            }
            SimpleTrace.trace("MyDAOFactory.getDao : returning class ["+dao.getClass().getName()+"]...");

         } catch (Exception e) {
            e.printStackTrace();
            throw new DAOException("MyDAOFactory.getDAO: Exception while getting DAO type : \n" + e.getMessage());
        }

        SimpleTrace.trace("MyDAOFactory.getDao : returning class ["+dao.getClass().getName()+"]...");
        return dao;
    }

}