package it.mokabyte.pattern.delegate;

// import javax.naming.*;  no, si usa ServiceLocator
// import javax.rmi.PrtableRemoteObject;  no, si usa ServiceLocator

import javax.ejb.CreateException;
import java.rmi.RemoteException;

import it.mokabyte.pattern.exception.BusinessDelegateException;

import it.mokabyte.pattern.locator.MyServiceLocator;
import it.mokabyte.pattern.exception.ServiceLocationException;

import it.mokabyte.pattern.util.JndiNames;

import it.mokabyte.pattern.ejb.session.stateful.FacadeHome;
import it.mokabyte.pattern.ejb.session.stateful.Facade;


 /**
  * <p>Title: MyBusinessDelegate </p>
  * <p>Description: pattern business delegate implementation</p>
  * <p>Copyright: Copyright (c) 2002</p>
  * <p>Company: </p>
  * @author unascribed
  * @version 1.0
  */
public class MyBusinessDelegate {

  /**
   * The EJB remote object for the RegisterSession service.
   */
  private Facade remote = null;

  /**
   * Constructor
   * @throws ServiceLocationException
   */
  public MyBusinessDelegate(String user_id) throws BusinessDelegateException {
    try {

      /* -----
      // Without ServiceLocater :
      Context ctx = new InitialContext();
      Object result = ctx.lookup(JndiNames.FACADE);
      FacadeHome home = (FacadeHome)PortableRemoteObject.narrow(result, FacadeHome.class);
      ------- */

      // With ServiceLocator is simple one statement ...
      FacadeHome home = (FacadeHome)MyServiceLocator.getInstance().getHome(JndiNames.FACADE,FacadeHome.class);

      System.out.println("MyBusinessDelegate::MyBusinessDelegate : : home class["+home.getClass().getName()+"] ...");
      System.out.println("MyBusinessDelegate::MyBusinessDelegate : : home ["+home+"] ...");

      remote = home.create(user_id);

      System.out.println("MyBusinessDelegate::MyBusinessDelegate : : remote class["+remote.getClass().getName()+"] ...");
      System.out.println("MyBusinessDelegate::MyBusinessDelegate : : remote ["+remote+"] ...");

    } catch (ServiceLocationException sle) {
        sle.printStackTrace();
        throw new BusinessDelegateException(sle);
    } catch (CreateException ce) {
        ce.printStackTrace();
        throw new BusinessDelegateException(ce.getMessage());
    }catch (Exception e) {
        e.printStackTrace();
        throw new BusinessDelegateException(e);
    }
  }

 /**
  * This method get the amount
  * @return
  * @throws BusinessDelegateException
  */
  public double getAmount() throws BusinessDelegateException {

    // Delegate to the EJB session bean
    try {
      return remote.getBalance();
    } catch (RemoteException re) {
        re.printStackTrace();
        throw new BusinessDelegateException(re);
    }

  }
}
