package it.mokabyte.pattern.ejb.session.stateful;

import java.rmi.*;
import javax.ejb.*;
import java.util.*;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.sql.*;
import javax.sql.*;

import it.mokabyte.pattern.util.SimpleTrace;

import it.mokabyte.pattern.valueobject.ProductOM;
import it.mokabyte.pattern.valueobject.ListProductModel;

import it.mokabyte.pattern.exception.ECommerceException;

/**
 * <p>Title: ShopperBean </p>
 * <p>Description: A simple eCommerce basket</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S.
 * @version 1.0 :
 */
public class ShopperBean implements SessionBean {

  /** The session context */
  private SessionContext sessionContext;

  /** Connection to DB released from DataSource */
  private Connection conn = null;

  /** The basket */
  private ListProductModel basket;

  /** Client ID */
  public String clientId;

  /** Amount to pay */
  private double amount;

 /**
  * @param id the client id
  * @throws CreateException
  */
  public void ejbCreate(String id) throws CreateException{
    	SimpleTrace.trace("ShopperBean.ejbCreate : id["+id+"]...");
    	clientId=id;
    	basket = new ListProductModel();
  }

  /**
   * Add an item to the basket. Update the amount of the basket.
   * @param prod the product to add to basket
   */
   public void addItem(ProductOM prod) {

    	SimpleTrace.trace("ShopperBean.addItem : prod["+prod+"]...");

    	basket.add(prod);
        this.amount += prod.getPrice();

        SimpleTrace.trace("ShopperBean.addItem :amount["+this.amount+"]...");
    	return;
  }

  /**
   * Remove the specified item form basket. Update the amount of the basket.
   * @param prod the product to remove from basket
   * @throws ECommerceException
   */
  public void removeItem(ProductOM prod) throws ECommerceException {

    	SimpleTrace.trace("ShopperBean.removeItem : prod["+prod+"]...");

        ProductOM p = (ProductOM)basket.get(prod);
        if(p == null)
          throw new ECommerceException("ShopperBean.removeItem : Object ["+prod+"] NOT FOUND in basket !!!");

   	this.amount -= prod.getPrice();
        basket.remove(p);

        SimpleTrace.trace("ShopperBean.removeItem :amount["+this.amount+"]...");
    	return;
  }

  /**
   * set the amount
   * @param newAmount to set
   */
  public void setAmount(double newAmount) {
    	amount = newAmount;
  }

  /**
   * @return amount
   */
  public double getAmount() {
    	return amount;
  }

   /**
    * @return the basket size
    */
   public int getBasketSize() {
  	System.out.println(new java.util.Date().toString() + "ShopperBean.getBasketSize() : return["+basket.size()+"]...");
    	return basket.size();
  }

  /* ************************* EJB Life Cycle methods  ********************************************/

  public void ejbRemove() {
	SimpleTrace.trace("ShopperBean.ejbRemove : ...");
  }

  public void ejbActivate() {
        SimpleTrace.trace("ShopperBean.ejbActivate : ...");
  }

  public void ejbPassivate() {
        SimpleTrace.trace("ShopperBean.ejbPassivate : ...");
  }

  public void setSessionContext(SessionContext context) {
	SimpleTrace.trace("ShopperBean.setSessionContext : context["+context+"]...");
    	sessionContext = context;
  }

}