package it.mokabyte.pattern.util;

//import jdk
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Enumeration;
import java.util.ArrayList;

//import junit
import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;

/**
 * Title:        <p> J2EEPatternTestResult
 * Description:  <p> DefinesClasse per il report automatico delle testunit
 * @author Dozio L.
 * @version 26-08-02
 */
public class J2EEPatternTestResult extends TestResult {

	/**
	 * property contenente i metodi/classi di test
	 */
	private java.util.ArrayList vcTestValue = new java.util.ArrayList();
	/**
	 * property per le classi di test gi verificate
	 */
	private java.util.ArrayList alreadyRead = new java.util.ArrayList();

	/**
	 * valori constanti per la generazione del messaggio ==> error
	 */
	private String ERROR = "error";
	/**
	 * valori constanti per la generazione del messaggio ==> ok
	 */
	private String OK = "ok";
	/**
	 * valori constanti per la generazione del messaggio ==> fail
	 */
	private String FAIL = "fail";

	/**
	 * costruttore
	 */
	public J2EEPatternTestResult(Test ts)
    {
     	if (ts != null)
     	{
     		this.initMethodTestCollection(ts);
     	}//else non faccio nulla e uso il comportamento di default nel toString

    }

    /**
     * metodo per l'inizializzazione della classe di TestResult
     *
     * @param ts --> classe di test da analizzare
     */
    private void initMethodTestCollection(Test ts){
//    	System.out.println("Elementi in initMethodTestCollection -->[" + ts.getClass() +"]");
    	if (ts instanceof TestSuite){
 	  		java.util.Enumeration enTest = ((TestSuite)ts).tests();
  //   	 	System.out.println("Count in suite -->[" + ((TestSuite)ts).testCount() +"]");
      		for (;enTest.hasMoreElements();)
      		{
        		Test o = (Test)enTest.nextElement();
    //    		System.out.println("Elementi in suite -->[" + o.getClass() +"]");
       			this.initMethodTestCollection(o);
       			if (!alreadyRead.contains(o.getClass().getName()))
       			{
      //			System.out.println("Classe gi verificata: " + o.getClass().getName());
       				alreadyRead.add(o.getClass().getName());
       			}
       		}
      	}
      	else{
      		// non  pi una suite suppongo sia la foglia dell'albero ==> aggiungo la key alla mappa
      		if(!alreadyRead.contains(ts.getClass().getName()))
      		{
      		//	System.out.println("in else quando ?" + ts.getClass());
      			this.initMapWithKeyAndMethod(ts.getClass());
      		}
     	}

    }

    /**
     * metodo per la costruzione della chiave per la verifica dei test corretti e dei test fail o error
     *
     * @param  testClass --> classe da analizzare con la reflection (trova i metodi di testNOME_TEST)
     */
    private void initMapWithKeyAndMethod(Class testClass){
    	Method[] methods = testClass.getDeclaredMethods();
    	for(int i=0; i < methods.length; i++){
    		if (Modifier.isPublic(methods[i].getModifiers())){
    			//costruisco la chiave
    			//System.out.println("metodo: ["+methods[i].getName()+"]");
    			if (isTestMethod(methods[i])){
	    			vcTestValue.add(this.makeKey(methods[i].getName(), testClass.getName()));
    			}
    			//else non aggiungo
    		}
    	}

    }


 	/**
 	 * metodo che verifica he il metodo individuato della classe sia effettivamente un metodo di test:
 	 * condizioni:
 	 * <li>non avere parametri in input; </li>
 	 * <li>avere un nome del tipo testNOME_TEST; </li>
 	 * <li>restituire VOID;</li>
 	 *
 	 * @param testMethod --> metodo da analizzare
 	 * @return true se le condizioni suddette sono verificate con logica AND
 	 */
    private boolean isTestMethod(Method testMethod){
    	String nameMethod = testMethod.getName();
    	Class[] parameters = testMethod.getParameterTypes();
    	Class returnType = testMethod.getReturnType();
    	return (parameters.length==0) && (nameMethod.startsWith("test")) && (returnType.equals(Void.TYPE));
    }


    /**
     * metodo che si occupa di realizzare la chiave di ricerca dei test gi analizzati
     *
     * @param methodName --> nome del metodo
     * @param className  --> nome della classe
     * @return stringa composta di chiave: nome_metodo(nome_classe)
     */
    private String makeKey(String methodName, String className){
    	StringBuffer sb = new StringBuffer(methodName);
    	sb.append("(").append(className).append(")");
    	//System.out.println("the key -->["+sb+"]");
    	return sb.toString();

    }

	/**
	 * metodo che sovrascrive il toString della classe madre e che genera la stringa per il report automatico, riportando
	 * la struttura in formato XML:
	 * @return string con il formato xml
	 */
	public String toString(){

		StringBuffer sb = new StringBuffer("<?xml version=\"1.0\"?>");
		sb.append("<TestResults>");
		if (vcTestValue==null)
		{
			//compaiono solo gli error e i fail
			for(int i=0; i<this.fFailures.size(); i++){
				String failure = fFailures.get(i).toString();
				String nome = failure.substring(0,failure.indexOf("("));
				String classe = failure.substring(failure.indexOf("(")+1,failure.indexOf(")"));
				sb.append("<TestResult ID=").append("\"fail").append(""+i).append("\">").append("<MethodName>").append(nome).append("</MethodName>");
				sb.append("<ClassName>").append(classe).append("</ClassName>");
				String msg = failure.substring(failure.indexOf(":")+1,failure.length());
                                msg = this.substitute(msg);// replace valori non validi per XML format
				sb.append("<MessageResult>").append(msg).append("</MessageResult>");

			}
			for(int i=0; i<this.fErrors.size(); i++){
				String errors = fErrors.get(i).toString();
				String nome = errors.substring(0,errors.indexOf("("));
				String classe = errors.substring(errors.indexOf("(")+1,errors.indexOf(")"));
			        sb.append("<TestResult ID=\"").append("error").append(""+i).append("\">").append("<MethodName>").append(nome).append("</MethodName>");
				sb.append("<ClassName>").append(classe).append("</ClassName>");
				String msg = errors.substring(errors.indexOf(":")+1,errors.length());
				msg = this.substitute(msg);// replace valori non validi per XML format
				sb.append("<MessageResult>").append(msg).append("</MessageResult>");


			}
                        sb.append("</TestResult>");
		}//end if
		else{
			//compaiono anche gli ok
			for(int i=0; i<this.fFailures.size(); i++){
				String failure = fFailures.get(i).toString();
				String keyRemove = failure.substring(0, failure.indexOf(":"));
                                vcTestValue.remove(keyRemove);
				sb.append(this.makeMessage(failure, this.FAIL,i));
			}
			for(int i=0; i<this.fErrors.size(); i++){
				String errors = fErrors.get(i).toString();
				String keyRemove = errors.substring(0, errors.indexOf(":"));
                                vcTestValue.remove(keyRemove);
				sb.append(this.makeMessage(errors, this.ERROR, i));

			}

			java.util.Iterator itHm = vcTestValue.iterator();
	        for (int i=0;itHm.hasNext();i++){
	        	String sOk = itHm.next().toString();
	        	sb.append(this.makeMessage(sOk, this.OK,i));

			}
		}//end else
                sb.append("</TestResults>");
		return sb.toString();

	}


	/**
	 * metodo che formatta il messaggio di report
	 *
	 * @param msgToProcess --> messaggio da elaborare
	 * @param constType    --> valore distintivo per l'elaborazione (error, fail, ok)
	 * @param i            --> indice di posizione delle classi di test e dei metodi di test
	 *
	 */
	private StringBuffer makeMessage(String msgToProcess, String constType, int i){
		StringBuffer sb = new StringBuffer();
		try{
			String nome = msgToProcess.substring(0,msgToProcess.indexOf("("));
			String classe = msgToProcess.substring(msgToProcess.indexOf("(")+1,msgToProcess.indexOf(")"));
			sb.append("<TestResult ID=\"").append(constType).append(""+i).append("\">").append("<MethodName>").append(nome).append("</MethodName>");
			sb.append("<ClassName>").append(classe).append("</ClassName>");
			String msg = null;
			if(!constType.equalsIgnoreCase(OK)){
				msg = msgToProcess.substring(msgToProcess.indexOf(":")+1,msgToProcess.length());
                               	msg = this.substitute(msg);
			}else msg ="OK";
			sb.append("<MessageResult>").append(msg).append("</MessageResult>");

		}
		catch(Exception e){
			//casomai cambiasse qualcosa nella stringa di rispota di junit
			String msg = this.substitute(msgToProcess);
			sb.append("<TestResult ID=").append(constType).append(""+i).append(">");
                        sb.append("<MethodName>").append(msg).append("</MethodName>");
                        sb.append("<ClassName>").append(msg).append("</ClassName>");
                        sb.append("<MessageResult>").append(msg).append("</MessageResult>");


		}
                sb.append("</TestResult>");
		return sb;

	}

        /**
         * metodo che si occupa di sostituire i parametri non valid per il formato XML
         * @param toSubstitute --> stringa da formattare
         * @return stringa correttamente formattata
         */

        private String substitute (String toSubstitute){
          StringBuffer sb = new StringBuffer();
          char[] a = toSubstitute.toCharArray();
          for (int i=0; i<a.length; i++){
            switch(a[i]){
              case '<':{
                sb.append("&lt;");
              };
              break;
               case '>':{
                 sb.append("&gt;");
               };
              break;
               case '"':{
                sb.append("&quot;");
               };
              break;
               case '&':{
                sb.append("&amp;");
               };
              break;
              default:{
                sb.append(a[i]);
              }
            }
          }
          return sb.toString();
        }




}

